# -*- coding: UTF-8 -*-
###########################################################################
# Eole NG - 2007
# Copyright Pole de Competence Eole  (Ministere Education - Academie Dijon)
# Licence CeCill  cf /root/LicenceEole.txt
# eole@ac-dijon.fr
###########################################################################

"""
Agent zephir pour l'étude des Statistiques Exim
"""

from zephir.monitor.agentmanager.agent import Agent
from zephir.monitor.agentmanager.data import HTMLData, TableData
from twisted.internet.utils import getProcessOutput
from zephir.monitor.agentmanager import status
from os.path import isfile


class EximStats(Agent):
    def __init__(self, name, **params):
        Agent.__init__(self, name, **params)
        self.status = status.OK()
        title1 = HTMLData("<h3>Etat du trafic<h3>")
        self.table = TableData([
            ('label', '', {'align':'center'}, None),
            ('quantity', 'Volume de données', {'align':'center'}, None),
            ('number', 'Nombre de messages', {'align':'center'}, None),
        ])
        title2 = HTMLData("<h3>Erreurs<h3>")
        self.table2 = TableData([
            ('label', '', {'align':'center'}, None),
            ('value', 'Valeur', {'align':'center'}, None),
        ])
        self.tdate = TableData([
            ('date', 'Période de mesure', {'align':'center'}, None),
        ])
        self.data = [self.tdate, title1, self.table, title2, self.table2]
        
    def _format_quantity(self, quantity):
        """
        L'option "-nvr" permet de toujours avoir le résulat en bytes
        Il n'y a plus qu'à le transformer en ko ou en Mo
        """
        quantity = float(quantity)/1024
        if quantity < 1000:
            return '%.1f ko' % quantity
        else:
            return '%.1f Mo' % (quantity/1024)
        
    def measure(self):
        """Utilisation de la commande eximstats"""
        """eximstats -nt -h0 -tnl -t0 -nr -nvr -q60 /var/log/exim4/mainlog"""
        args = ['-nt', '-h0', '-tnl', '-t0', '-nr', '-nvr', '-q60']
        
        f_log1 = '/var/log/exim4/mainlog'
        f_log2 = '/var/log/exim4/mainlog.1'

        if not isfile(f_log1) and not isfile(f_log2) :
            # si pas fichier on court à la catastrophe :)
            return self.no_measure()
        
        if isfile(f_log1) :
            args.append(f_log1)            
        if isfile(f_log2) :
            args.append(f_log2)
            
        res = getProcessOutput("/usr/sbin/eximstats",
                             args = args,
                             env = {'LC_ALL': 'C'})
                             
        res.addCallback(self.measure_process)
        return res


    def measure_process(self,result):
       # étude du résultat
       res1 = []
       res2 = []
       date = []
       Queue = False
       if result.startswith('****'):
           # No valid log lines read
           return self.no_measure()
       for ligne in result.splitlines():
           # dernier !
           if ligne.startswith('Errors encountered:') :
               # en première ligne
               # L.insert(index, object) -- insert object before index
               res2.insert (0, {'label' : 'Nombre d\'erreurs', 'value' : ligne[20:]} )
           if ligne.startswith('Exim statistics from') : 
               l = ligne.split()
               deb = l[3].split('-')
               fin = l[6].split('-')
               date.append ({'date' : 'du %s/%s/%s au %s/%s/%s' % (deb[2],deb[1],deb[0],fin[2],fin[1],fin[0])})
           # 1er
           elif ligne.startswith('  Received '):
               l = ligne.split()
               res1.append ({'label' : 'Réception', 'quantity' : self._format_quantity(l[1]), 'number' : l[2] })
               res2.append ({'label' : 'Pourcentage de mails en attente', 'value' : l[5] })
               res2.append ({'label' : 'Pourcentage de mails en erreur', 'value' : l[7] })

           # 2ème
           elif ligne.startswith('  Delivered'):
               l = ligne.split()
               res1.append ({'label' : 'Envoi', 'quantity' : self._format_quantity(l[1]), 'number' : l[2] })
           
           # 3ème
           elif ligne.startswith('Under   1m') and Queue==False:
               # tour de passe-passe car la ligne y est 2 fois
               # on prend la première !
               Queue = True
               l = ligne.split()
               res2.insert (0, {'label' : 'Mails ayant passé plus de 1 minute en queue', 
                             'value' : '%.1f%%' % ( float(100) - float(l[3][:-1]) ) })
               
       return { 'statistics'  : res1,
                'statistics2' : res2,
                'statdate' : date
              }


    def no_measure(self):
        return { 'statistics'  : [{'label' : '', 'quantity' : '0', 'number' : '0'}],
                 'statistics2' : [{'label' : '', 'value' : '---'}],
                 'statdate'    : [{'date'  : '---' }],
               }
               

    def write_data(self):
        Agent.write_data(self)
        if self.last_measure is not None:
            self.table.table_data = self.last_measure.value['statistics']
            self.table2.table_data = self.last_measure.value['statistics2']
            self.tdate.table_data = self.last_measure.value['statdate']
            
    def check_status(self):
        return self.status
 
