# -*- coding: UTF-8 -*-
###########################################################################
# Eole NG - 2007
# Copyright Pole de Competence Eole  (Ministere Education - Academie Dijon)
# Licence CeCill  cf /root/LicenceEole.txt
# eole@ac-dijon.fr
###########################################################################

"""
Agent zephir surveillant le fonctionnement du service horus_frontend
"""

import os
from creole import parsedico
from twisted.internet import defer
from twisted.internet.utils import getProcessOutput, getProcessValue

from zephir.monitor.agentmanager.agent import Agent
from zephir.monitor.agentmanager.data import TableData
from zephir.monitor.agentmanager.util import status_to_img, boolean_to_onoff
from zephir.monitor.agentmanager import status

def compare_hostports(hp1, hp2):
    h1, p1 = hp1.split(':')
    h2, p2 = hp2.split(':')
    result = cmp(h1, h2) # alpha comparison on hosts
    if result != 0:
        return result
    else: # numeric comparison on ports
        return cmp(int(p1), int(p2))

class Frontend(Agent):

    def __init__(self, name, **params):

        Agent.__init__(self, name, **params)
        try:
            d = parsedico.parse_dico()
            self.adresse_eth0 = d['adresse_ip_eth0']
        except:
            self.adresse_eth0 = "127.0.0.1"
        
        self.table = TableData([
            ('status', "état", {'align':'center'}, status_to_img), 
            ('description', "Description", {'align':'left'}, None),
            ('host', "Hôte", {'align':'left'}, None),
            ('port', "Port", {'align':'left'}, None) ])
        self.data = [self.table]

    def measure(self):
        res = os.system('/etc/init.d/horus_frontend status &> /dev/null')
        if res == 0:
            result = True
        else:
            result = False
            
        status = boolean_to_onoff(result)
        mesures=[{ 'description': 'serveur du frontend LDAP',
                   'host': self.adresse_eth0,
                   'port': '7080',
                   'status': status }]
        return {'services': mesures}

    def write_data(self):
        Agent.write_data(self)
        if self.last_measure is not None:
            self.table.table_data = self.last_measure.value['services']

    def check_status(self):
        """remonte une erreur si un des services est tombé"""
        if self.last_measure is not None:
            for service in self.last_measure.value['services']:
                if service['status'] != 'On':
                    return status.Error()
        else:
            # pas de mesure connue
            return status.Unknown()
        return status.OK()

